<?php

namespace Drupal\ip_info\Controller;

use Drupal\ban\BanIpManagerInterface;
use Drupal\dblog\Controller\DbLogController;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\PagerSelectExtender;
use Drupal\Core\Database\Query\TableSortExtender;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Link;
use Drupal\Core\Site\Settings;
use Drupal\Core\Url;
use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\Unicode;

/**
 * Controller for IP address.
 */
class IpController extends DbLogController {

  /**
   * The ban IP manager.
   *
   * @var \Drupal\ban\BanIpManagerInterface
   */
  protected $ipManager;

  /**
   * Constructs a new IpController object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   * @param \Drupal\ban\BanIpManagerInterface $ip_manager
   *   The ban IP manager.
   */
  public function __construct(Connection $database, ModuleHandlerInterface $module_handler, DateFormatterInterface $date_formatter, FormBuilderInterface $form_builder, BanIpManagerInterface $ip_manager) {
    parent::__construct($database, $module_handler, $date_formatter, $form_builder);
    $this->ipManager = $ip_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function overview($ip_address) {

    // Test, if the IP address is banned.
    if ($this->ipManager->isBanned($ip_address)) {
      $build['dblog_filter_form']['#prefix'] = '<p>' . $this->t('The IP address is banned.') . ' ';
      // Find the ID of the banned IP.
      $bans = $this->ipManager->findAll()->fetchAll();
      foreach ($bans as $ban) {
        if ($ban->ip == $ip_address) {
          $ban_id = $ban->iid;
          break;
        }
      }
      // Add unban link.
      if (isset($ban_id)) {
        $build['dblog_filter_form']['#prefix'] .= Link::fromTextAndUrl($this->t('Unban this IP address'), Url::fromRoute('ban.delete', ['ban_id' => $ban_id]))->toString() . '.</p>';
      }
      else {
        $build['dblog_filter_form']['#prefix'] .= '</p>';
      }
    }
    else {
      $build['dblog_filter_form']['#prefix'] = '<p>' . $this->t('The IP address is not banned.') . ' ';
      // Add ban link.
      $build['dblog_filter_form']['#prefix'] .= Link::fromTextAndUrl($this->t('Ban this IP address'), Url::fromRoute('ban.admin_page', ['default_ip' => $ip_address]))->toString() . '.</p>';
    }

    // DNS Lookup.
    $lookup_services = [
      'ipinfo.io' => 'https://ipinfo.io/' . $ip_address,
      'iplocation.net' => 'https://www.iplocation.net/ip-lookup/' . $ip_address,
      'ip2location.com' => 'https://www.ip2location.com/demo/' . $ip_address,
      'keycdn.com' => 'https://tools.keycdn.com/geo?host=' . $ip_address,
    ];
    $lookup_services_links = '';
    foreach ($lookup_services as $service => $url) {
      $lookup_services_links .= '<a href="' . $url . '">' . $service . '</a>, ';
    }
    // Remove the last comma.
    $lookup_services_links = rtrim($lookup_services_links, ', ');
    $build['dblog_filter_form']['#prefix'] .= '<p>' . $this->t('Lookup the IP:') . ' ' . $lookup_services_links . '</p>';

    // Get IP data from IPinfo.io API.
    if (
      ($ip_info_settings = Settings::get('ip_info')) &&
      array_key_exists('ipinfo_io_token', $ip_info_settings) &&
      $ipinfo_io_token = $ip_info_settings['ipinfo_io_token']
    ) {
      $ipinfo_io_url = 'https://ipinfo.io/' . $ip_address . '?token=' . $ipinfo_io_token;
      $ipinfo_io_response = file_get_contents($ipinfo_io_url);
      $ipinfo_io_data = json_decode($ipinfo_io_response, TRUE);
      if ($ipinfo_io_data) {
        $ipinfo_data_array = [];
        if (array_key_exists('country', $ipinfo_io_data)) {
          $ipinfo_data_array[] = ['label' => $this->t('Country'), 'value' => $ipinfo_io_data['country']];
        }
        if (array_key_exists('region', $ipinfo_io_data)) {
          $ipinfo_data_array[] = ['label' => $this->t('Region'), 'value' => $ipinfo_io_data['region']];
        }
        if (array_key_exists('city', $ipinfo_io_data)) {
          $ipinfo_data_array[] = ['label' => $this->t('City'), 'value' => $ipinfo_io_data['city']];
        }
        if (array_key_exists('org', $ipinfo_io_data)) {
          $ipinfo_data_array[] = ['label' => $this->t('Organization'), 'value' => $ipinfo_io_data['org']];
        }
        if (array_key_exists('hostname', $ipinfo_io_data)) {
          $ipinfo_data_array[] = ['label' => $this->t('Hostname'), 'value' => $ipinfo_io_data['hostname']];
        }

        if (!empty($ipinfo_data_array)) {
          $build['ipinfo_label'] = [
            '#type' => 'markup',
            '#markup' => '<h3>' . $this->t('IPinfo.io data') . '</h3>',
          ];
          $ipinfo_rows = [];
          foreach ($ipinfo_data_array as $ipinfo_data) {
            $ipinfo_rows[] = ['data' => [$ipinfo_data['label'], $ipinfo_data['value']]];
          }
          $build['ipinfo_table'] = [
            '#type' => 'table',
            '#rows' => $ipinfo_rows,
            '#attributes' => ['id' => 'admin-ipinfo', 'class' => ['admin-list']],
            '#empty' => $this->t('No IP information available.'),
          ];
        }
      }
    }

    // Get IP data from ipapi.is API.
    if (
      ($ip_info_settings = Settings::get('ip_info')) &&
      array_key_exists('ipapi_is_key', $ip_info_settings) &&
      $ipapi_is_key = $ip_info_settings['ipapi_is_key']
    ) {
      $ipapi_is_url = 'https://api.ipapi.is/?q=' . $ip_address . '&key=' . $ipapi_is_key;
      $ipapi_is_response = file_get_contents($ipapi_is_url);
      $ipapi_is_data = json_decode($ipapi_is_response, TRUE);
      if ($ipapi_is_data) {
        $ipapi_is_data_array = [];
        if (array_key_exists('location', $ipapi_is_data)) {
          $location_info = "";
          if (array_key_exists('city', $ipapi_is_data['location'])) {
            $location_info .= $ipapi_is_data['location']['city'];
          }
          if (array_key_exists('zip', $ipapi_is_data['location'])) {
            $location_info .= " (" . $ipapi_is_data['location']['zip'] . ")";
          }
          if (array_key_exists('state', $ipapi_is_data['location'])) {
            $location_info .= ", " . $ipapi_is_data['location']['state'];
          }
          if (array_key_exists('country', $ipapi_is_data['location'])) {
            $location_info .= ", " . $ipapi_is_data['location']['country'];
          }
          $ipapi_is_data_array[] = ['label' => $this->t('Location'), 'value' => $location_info];
        }
        if (array_key_exists('company', $ipapi_is_data)) {
          $company_info = "";
          if (array_key_exists('name', $ipapi_is_data['company'])) {
            $company_info .= $ipapi_is_data['company']['name'];
          }
          if (array_key_exists('type', $ipapi_is_data['company'])) {
            $company_info .= " - " . $ipapi_is_data['company']['type'];
          }
          if (array_key_exists('domain', $ipapi_is_data['company'])) {
            $company_info .= " (" . $ipapi_is_data['company']['domain'] . ")";
          }
          if (array_key_exists('abuser_score', $ipapi_is_data['company'])) {
            $company_info .= $this->t("; abuser score: @score", ['@score' => $ipapi_is_data['company']['abuser_score']]);
          }
          $ipapi_is_data_array[] = ['label' => $this->t('Company'), 'value' => $company_info];
        }
        if (array_key_exists('datacenter', $ipapi_is_data)) {
          $data_center_info = "";
          if (array_key_exists('datacenter', $ipapi_is_data['datacenter'])) {
            $data_center_info .= $ipapi_is_data['datacenter']['datacenter'];
          }
          if (array_key_exists('domain', $ipapi_is_data['datacenter'])) {
            $data_center_info .= " (" . $ipapi_is_data['datacenter']['domain'] . ")";
          }
          $ipapi_is_data_array[] = ['label' => $this->t('Data center'), 'value' => $data_center_info];
        }

        $ip_properties = [
          "is_bogon" => $this->t("Bogon"),
          "is_mobile" => $this->t("Mobile"),
          "is_satellite" => $this->t("Satellite"),
          "is_crawler" => $this->t("Crawler"),
          "is_datacenter" => $this->t("Datacenter"),
          "is_tor" => $this->t("Tor"),
          "is_proxy" => $this->t("Proxy"),
          "is_vpn" => $this->t("VPN"),
          "is_abuser" => $this->t("Abuser"),
        ];
        $ip_is = "";
        foreach ($ip_properties as $ip_property => $ip_property_label) {
          if ($ipapi_is_data[$ip_property]) {
            if (!empty($ip_is)) {
              $ip_is .= ", ";
            }
            $ip_is .= $ip_property_label;
          }
        }
        if (!empty($ip_is)) {
          $ipapi_is_data_array[] = ['label' => $this->t('The IP is'), 'value' => $ip_is];
        }

        if (!empty($ipapi_is_data_array)) {
          $build['ipapi_label'] = [
            '#type' => 'markup',
            '#markup' => '<h3>' . $this->t('ipapi.is data') . '</h3>',
          ];
          $ipapi_rows = [];
          foreach ($ipapi_is_data_array as $ipapi_data) {
            $ipapi_rows[] = ['data' => [$ipapi_data['label'], $ipapi_data['value']]];
          }
          $build['ipapi_table'] = [
            '#type' => 'table',
            '#rows' => $ipapi_rows,
            '#attributes' => ['id' => 'admin-ipapi', 'class' => ['admin-list']],
            '#empty' => $this->t('No IP information available.'),
          ];
        }
      }
    }

    $build['dblog_label'] = [
      '#type' => 'markup',
      '#markup' => '<h3>' . $this->t('Log messages') . '</h3>',
    ];

    $dblog_rows = [];

    $classes = static::getLogLevelClassMap();

    $dblog_header = [
      // Icon column.
      '',
      [
        'data' => $this->t('Type'),
        'field' => 'w.type',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Date'),
        'field' => 'w.wid',
        'sort' => 'desc',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      $this->t('Message'),
      [
        'data' => $this->t('User'),
        'field' => 'ufd.name',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
    ];

    $query = $this->database->select('watchdog', 'w')
      ->extend(PagerSelectExtender::class)
      ->extend(TableSortExtender::class);
    $query->fields('w', [
      'wid',
      'uid',
      'severity',
      'type',
      'hostname',
      'timestamp',
      'message',
      'variables',
      'link',
    ]);
    $query->leftJoin('users_field_data', 'ufd', '[w].[uid] = [ufd].[uid]');

    $result = $query
      ->limit(50)
      ->condition('hostname', $ip_address)
      ->orderByHeader($dblog_header)
      ->execute();

    foreach ($result as $dblog) {
      $message = $this->formatMessage($dblog);
      if ($message && isset($dblog->wid)) {
        $title = Unicode::truncate(Html::decodeEntities(strip_tags($message)), 256, TRUE, TRUE);
        $log_text = Unicode::truncate($title, 56, TRUE, TRUE);
        // The link generator will escape any unsafe HTML entities in the final
        // text.
        $message = Link::fromTextAndUrl($log_text, new Url('dblog.event', ['event_id' => $dblog->wid], [
          'attributes' => [
            // Provide a title for the link for useful hover hints. The
            // Attribute object will escape any unsafe HTML entities in the
            // final text.
            'title' => $title,
          ],
        ]))->toString();
      }
      $username = [
        '#theme' => 'username',
        '#account' => $this->userStorage->load($dblog->uid),
      ];
      $dblog_rows[] = [
        'data' => [
          // Cells.
          ['class' => ['icon']],
          $dblog->type,
          $this->dateFormatter->format($dblog->timestamp, 'short'),
          $message,
          ['data' => $username],
          ['data' => ['#markup' => $dblog->link]],
        ],
        // Attributes for table row.
        'class' => [Html::getClass('dblog-' . $dblog->type), $classes[$dblog->severity]],
      ];
    }

    $build['dblog_table'] = [
      '#type' => 'table',
      '#header' => $dblog_header,
      '#rows' => $dblog_rows,
      '#attributes' => ['id' => 'admin-dblog', 'class' => ['admin-dblog']],
      '#empty' => $this->t('No log messages available.'),
      '#attached' => [
        'library' => ['dblog/drupal.dblog'],
      ],
    ];
    $build['dblog_pager'] = ['#type' => 'pager'];

    return $build;

  }

  /**
   * {@inheritdoc}
   */
  public function eventDetails($event_id) {
    $build = parent::eventDetails($event_id);
    $ip_address = $build["dblog_table"]["#rows"][7][1];
    $build["dblog_table"]["#rows"][7][1] = Link::fromTextAndUrl($ip_address, Url::fromRoute('ip_info.ip', ['ip_address' => $ip_address]));
    return $build;
  }

  /**
   * Generates the page title.
   *
   * @param string $ip_address
   *   The IP address from the path.
   *
   * @return string
   *   The page title.
   */
  public function title($ip_address) {
    return 'IP Address Info for: ' . $ip_address;
  }

}
